/**********************************************************************
 * Copyright (c) 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * IBM - Initial API and implementation
 *
 * $Id$
 **********************************************************************/
package org.eclipse.cosmos.dr.reports.oda;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.tptp.platform.provisional.dms.DataAccessFactory;
import org.eclipse.tptp.platform.provisional.dms.IDataContext;
import org.eclipse.tptp.platform.provisional.dms.IDataOutputFormat;
import org.eclipse.tptp.platform.provisional.dms.IDataProperty;
import org.eclipse.tptp.platform.provisional.dms.IDataSource;
import org.eclipse.tptp.platform.provisional.dms.resource.IResource;
import org.eclipse.tptp.platform.provisional.dms.resource.IResourceDataService;

/**
 * An data accessor class that is used to retrieve the resources from a set of
 * topologies.
 * 
 * @author slavescu
 * 
 */

public class ResourcesDataAccessor {
	public static String TOPOLOGY_PRODUCTION = "//" + IResourceDataService.id
			+ ":topology[1]";

	// public static String PRODUCTION_MACHINES_REFS = "//" +
	// IResourceDataService.id + ":topology[0]//uri";

	// public static String PRODUCTION_MACHINES_REFS =
	// "/smlif:model[0]/smlif:phenics[2]/smlif:document[0]/smlif:data[1]/s:topology[0]//sml:uri";
	public static String PRODUCTION_MACHINES_REFS = "//s:topology[@name=\"Production\"]//it:hosts//sml:uri";

	private static String SML_RESOURCES_FOLDER = "file:///c|/COSMOS-Demo-EclipseCon2007/eclipse/demo/org.eclipse.cosmos.examples.eclipsecon07.repository/topologies/";

	public List getTopologies(IDataSource dataSource,
			IDataOutputFormat dataOutputFormat) {

		if (dataSource == null) {
			dataSource = DataAccessFactory.getResourceDataService()
					.createDataSource(
							SML_RESOURCES_FOLDER + "Topologies_sml-if.xml");
		}
		if (dataOutputFormat == null) {
			dataOutputFormat = (IResource) DataAccessFactory
					.getResourceDataService().createDataOutputFormat(
							IResource.class);
			IDataProperty dataProperty = DataAccessFactory
					.getResourceDataService().createDataProperty("@name",
							"string", null);
			dataOutputFormat.getDynamicProperties().add(dataProperty);
			dataProperty = DataAccessFactory.getResourceDataService()
					.createDataProperty("@description", "string", null);
			dataOutputFormat.getDynamicProperties().add(dataProperty);
			dataOutputFormat
					.setDataContext(DataAccessFactory.getResourceDataService()
							.createDataContext("//s:topology"));
		}

		return getResources(dataSource, dataOutputFormat);
	}

	public List getMachinesRef(IDataSource dataSource,
			IDataOutputFormat dataOutputFormat) {
		if (dataSource == null) {
			dataSource = DataAccessFactory.getResourceDataService()
					.createDataSource(
							SML_RESOURCES_FOLDER + "Topologies_sml-if.xml");
		}

		if (dataOutputFormat == null) {
			dataOutputFormat = (IResource) DataAccessFactory
					.getResourceDataService().createDataOutputFormat(
							IResource.class);
		}

		if (dataOutputFormat.getDataContext() == null) {
			dataOutputFormat.setDataContext(DataAccessFactory
					.getResourceDataService().createDataContext(
							PRODUCTION_MACHINES_REFS));
		}
		return getRefs(dataSource, dataOutputFormat);

	}

	public List getMachines(IDataSource dataSource,
			IDataOutputFormat dataOutputFormat) {
		List machineRefs = getMachinesRef(dataSource, dataOutputFormat);
		if (dataSource == null) {
			dataSource = DataAccessFactory.getResourceDataService()
					.createDataSource(
							SML_RESOURCES_FOLDER + "Topologies_sml-if.xml");
		}

		if (dataOutputFormat == null) {
			dataOutputFormat = (IResource) DataAccessFactory
					.getResourceDataService().createDataOutputFormat(
							IResource.class);
		}

		IDataProperty dataProperty = DataAccessFactory.getResourceDataService()
				.createDataProperty("@name", "string", null);
		dataOutputFormat.getDynamicProperties().add(dataProperty);
		dataProperty = DataAccessFactory.getResourceDataService()
				.createDataProperty("@description", "string", null);
		dataOutputFormat.getDynamicProperties().add(dataProperty);

		List machines = new ArrayList();
		for (int i = 0; i < machineRefs.size(); i++) {
			dataOutputFormat.setDataContext(DataAccessFactory
					.getResourceDataService().createDataContext(
							((IResource) machineRefs.get(i)).getDataContext()
									.getURI()));
			machines.addAll(getResources(dataSource, dataOutputFormat));
		}

		return machines;

	}

	public List getRefs(IDataSource dataSource,
			IDataOutputFormat dataOutputFormat) {
		return (List) (DataAccessFactory.getDataAccessController()
				.executeCommand(
						null,
						DataAccessFactory.getResourceDataService().getRefs(
								dataSource, dataOutputFormat), true, null)
				.getResult());
	}

	public List getResources(IDataSource dataSource,
			IDataOutputFormat dataOutputFormat) {
		return (List) (DataAccessFactory.getDataAccessController()
				.executeCommand(
						null,
						DataAccessFactory.getResourceDataService()
								.getResources(dataSource, dataOutputFormat),
						true, null).getResult());
	}

	private void testGetToplogies() {

		List topoInst = getTopologies(null, null);
		for (int count = 0; count < topoInst.size(); count++) {
			IDataProperty nameProp = ((IDataProperty) ((IResource) topoInst
					.get(count)).getDynamicProperties().get(0));
			System.out.println(nameProp.getName() + "[" + count + "]="
					+ nameProp.getValue());
			IDataProperty descriptionProp = ((IDataProperty) ((IResource) topoInst
					.get(count)).getDynamicProperties().get(1));
			System.out.println(descriptionProp.getName() + "=[" + count + "]"
					+ descriptionProp.getValue());
			System.out.println("contextURI["
					+ count
					+ "]="
					+ ((IResource) topoInst.get(count)).getDataContext()
							.getURI());
			System.out.println("refs["
					+ count
					+ "]="
					+ getMachinesRef(
							null,
							createIResource(((IResource) topoInst.get(count))
									.getDataContext().getURI()
									+ "//it:hosts//sml:uri")).size());
		}
	}

	public static void main(String[] args) {
		ResourcesDataAccessor resourcesDataAccessor = new ResourcesDataAccessor();
		resourcesDataAccessor.testGetToplogies();
		resourcesDataAccessor.testGetMachines();
	}

	private void testGetMachines() {
		List machines = getMachines(null,
				createIResource(PRODUCTION_MACHINES_REFS));
		System.out.println("Machines for " + TOPOLOGY_PRODUCTION);
		for (int count = 0; count < machines.size(); count++) {
			IDataProperty nameProp = ((IDataProperty) ((IResource) machines
					.get(count)).getDynamicProperties().get(0));
			System.out.println(nameProp.getName() + "[" + count + "]="
					+ nameProp.getValue());
			IDataProperty descriptionProp = ((IDataProperty) ((IResource) machines
					.get(count)).getDynamicProperties().get(1));
			System.out.println(descriptionProp.getName() + "[" + count + "]="
					+ descriptionProp.getValue());
			System.out.println("contextURI["
					+ count
					+ "]="
					+ ((IResource) machines.get(count)).getDataContext()
							.getURI());

		
		}
//		ResourcesDataAccessor dataAccessor = this;
//		IDataOutputFormat dataOutputFormat = null;
////		dataAccessor.getMachinesRef(null,dataAccessor.createIResource(((IResource) nodeInst.get(count)).getDataContext().getURI()+ "//sml:uri")).size();
//		dataOutputFormat = dataAccessor.createIResource(params["parentId"]);
//		dataOutputFormat.getDynamicProperties().add(dataAccessor.createDataProperty("@name","string",null));
//		dataOutputFormat.getDynamicProperties().add(dataAccessor.createDataProperty("@description","string",null));
//		dataOutputFormat.getDynamicProperties().add(dataAccessor.createDataProperty("@loggingType","string",null));
//		dataOutputFormat.getDynamicProperties().add(dataAccessor.createDataProperty("@logingId","string",null));
//		resource = dataAccessor.getResources(null,dataOutputFormat).get(0);
//		propertyInst = resource.getDynamicProperties();

	}

	public static IDataContext createDataContext(String uri) {
		return DataAccessFactory.getResourceDataService()
				.createDataContext(uri);
	}

	public static IDataProperty createDataProperty(String name,String type,String value) {
		return DataAccessFactory.getResourceDataService()
				.createDataProperty(name, type, value);
	}

	public static IResource createIResource(String uri) {

		IResource res = (IResource) DataAccessFactory.getResourceDataService()
				.createDataOutputFormat(IResource.class);
		res.setDataContext(DataAccessFactory.getResourceDataService()
				.createDataContext(uri));
		return res;
	}

}
